@@ -9,66 +9,52 @@
 // Convert analyzed data to javascript. Everything has already been calculated
 // before this stage, which just does the final conversion to JavaScript.
 
-// Handy sets
-
-var STRUCT_LIST = set('struct', 'list');
-
 var addedLibraryItems = {};
-var asmLibraryFunctions = [];
-
-var allExternPrimitives = ['Math_floor', 'Math_abs', 'Math_sqrt', 'Math_pow',
-  'Math_cos', 'Math_sin', 'Math_tan', 'Math_acos', 'Math_asin', 'Math_atan',
-  'Math_atan2', 'Math_exp', 'Math_log', 'Math_ceil', 'Math_imul', 'Math_min',
-  'Math_max', 'Math_clz32', 'Math_fround',
-  'Int8Array', 'Uint8Array', 'Int16Array', 'Uint16Array', 'Int32Array',
-  'Uint32Array', 'Float32Array', 'Float64Array'];
-
-// Specifies the set of referenced built-in primitives such as Math.max etc.
-var usedExternPrimitives = {};
-
-var SETJMP_LABEL = -1;
-
-var INDENTATION = ' ';
-
-var functionStubSigs = {};
 
 // Some JS-implemented library functions are proxied to be called on the main browser thread, if the Emscripten runtime is executing in a Web Worker.
 // Each such proxied function is identified via an ordinal number (this is not the same namespace as function pointers in general).
 var proxiedFunctionTable = ["null" /* Reserve index 0 for an undefined function*/];
 
-// proxiedFunctionInvokers contains bodies of the functions that will perform the proxying. These
-// are generated in a map to keep track which ones have already been emitted, to avoid outputting duplicates.
-// map: pair(sig, syncOrAsync) -> function body
-var proxiedFunctionInvokers = {};
-
-// We include asm2wasm imports if the trap mode is 'js' (to call out to JS to do some math stuff).
-// However, we always need some of them (like the frem import because % is in asm.js but not in wasm).
-// But we can avoid emitting all the others in many cases.
-var NEED_ALL_ASM2WASM_IMPORTS = BINARYEN_TRAP_MODE == 'js';
-
 // Used internally. set when there is a main() function.
 // Also set when in a linkable module, as the main() function might
 // arrive from a dynamically-linked library, and not necessarily
 // the current compilation unit.
 // Also set for STANDALONE_WASM since the _start function is needed to call
 // static ctors, even if there is no user main.
-var HAS_MAIN = ('_main' in IMPLEMENTED_FUNCTIONS) || MAIN_MODULE || SIDE_MODULE || STANDALONE_WASM;
-
-WEAK_DECLARES = set(WEAK_DECLARES);
+var HAS_MAIN = ('_main' in IMPLEMENTED_FUNCTIONS) || MAIN_MODULE || STANDALONE_WASM;
 
 // Mangles the given C/JS side function name to assembly level function name (adds an underscore)
 function mangleCSymbolName(f) {
   return f[0] == '$' ? f.substr(1) : '_' + f;
 }
 
-// Reverses C/JS name mangling: _foo -> foo, and foo -> $foo.
-function demangleCSymbolName(f) {
-  return f[0] == '_' ? f.substr(1) : '$' + f;
+// Splits out items that pass filter. Returns also the original sans the filtered
+function splitter(array, filter) {
+  var splitOut = array.filter(filter);
+  var leftIn = array.filter((x) => !filter(x));
+  return { leftIn: leftIn, splitOut: splitOut };
 }
 
-function isJsLibraryConfigIdentifier(ident) {
-  return ident.endsWith('__sig') || ident.endsWith('__proxy') || ident.endsWith('__asm') || ident.endsWith('__inline')
-   || ident.endsWith('__deps') || ident.endsWith('__postset') || ident.endsWith('__docs') || ident.endsWith('__import');
+// Functions that start with '$' should not be imported to asm.js/wasm module.
+// They are intended to be exclusive to JS code only.
+function isJsOnlyIdentifier(ident) {
+  return ident[0] == '$';
+}
+
+function escapeJSONKey(x) {
+  if (/^[\d\w_]+$/.exec(x) || x[0] === '"' || x[0] === "'") return x;
+  assert(x.indexOf("'") < 0, 'cannot have internal single quotes in keys: ' + x);
+  return "'" + x + "'";
+}
+
+function stringifyWithFunctions(obj) {
+  if (typeof obj === 'function') return obj.toString();
+  if (obj === null || typeof obj !== 'object') return JSON.stringify(obj);
+  if (Array.isArray(obj)) {
+    return '[' + obj.map(stringifyWithFunctions).join(',') + ']';
+  } else {
+    return '{' + keys(obj).map((key) => escapeJSONKey(key) + ':' + stringifyWithFunctions(obj[key])).join(',') + '}';
+  }
 }
 
 // JSifier
@@ -85,8 +71,7 @@ function JSify(data, functionsOnly) {
 
     var libFuncsToInclude;
     if (INCLUDE_FULL_LIBRARY) {
-      assert(!SIDE_MODULE, 'Cannot have both INCLUDE_FULL_LIBRARY and SIDE_MODULE set.')
-      libFuncsToInclude = (MAIN_MODULE || SIDE_MODULE) ? DEFAULT_LIBRARY_FUNCS_TO_INCLUDE.slice(0) : [];
+      libFuncsToInclude = MAIN_MODULE ? DEFAULT_LIBRARY_FUNCS_TO_INCLUDE.slice(0) : [];
       for (var key in LibraryManager.library) {
         if (!isJsLibraryConfigIdentifier(key)) {
           libFuncsToInclude.push(key);
@@ -95,9 +80,10 @@ function JSify(data, functionsOnly) {
     } else {
       libFuncsToInclude = DEFAULT_LIBRARY_FUNCS_TO_INCLUDE;
     }
-    libFuncsToInclude.forEach(function(ident) {
+    libFuncsToInclude.forEach((ident) => {
       data.functionStubs.push({
-        ident: mangleCSymbolName(ident)
+        identOrig: ident,
+        identMangled: mangleCSymbolName(ident)
       });
     });
   }
@@ -107,22 +93,18 @@ function JSify(data, functionsOnly) {
     // line endings \r\n. This is undesirable, since line endings are managed in the form \n in the output for binary file writes, so
     // make sure the endings are uniform.
     snippet = snippet.toString().replace(/\r\n/gm,"\n");
-    assert(snippet.indexOf('XXX missing C define') == -1,
-           'Trying to include a library function with missing C defines: ' + finalName + ' | ' + snippet);
 
     // name the function; overwrite if it's already named
     snippet = snippet.replace(/function(?:\s+([^(]+))?\s*\(/, 'function ' + finalName + '(');
-    // Apply special js library debug modes
-    if (!LibraryManager.library[ident + '__asm']) {
-      // apply LIBRARY_DEBUG if relevant
-      if (LIBRARY_DEBUG) {
-        snippet = modifyFunction(snippet, function(name, args, body) {
-          return 'function ' + name + '(' + args + ') {\n' +
-                 'var ret = (function() { if (runtimeDebug) err("[library call:' + finalName + ': " + Array.prototype.slice.call(arguments).map(prettyPrint) + "]");\n' +
-                  body +
-                  '}).apply(this, arguments); if (runtimeDebug && typeof ret !== "undefined") err("  [     return:" + prettyPrint(ret)); return ret; \n}\n';
-        });
-      }
+
+    // apply LIBRARY_DEBUG if relevant
+    if (LIBRARY_DEBUG) {
+      snippet = modifyFunction(snippet, (name, args, body) => {
+        return 'function ' + name + '(' + args + ') {\n' +
+               'var ret = (function() { if (runtimeDebug) err("[library call:' + finalName + ': " + Array.prototype.slice.call(arguments).map(prettyPrint) + "]");\n' +
+                body +
+                '}).apply(this, arguments); if (runtimeDebug && typeof ret !== "undefined") err("  [     return:" + prettyPrint(ret)); return ret; \n}\n';
+      });
     }
     return snippet;
   }
@@ -132,31 +114,32 @@ function JSify(data, functionsOnly) {
     // In LLVM, exceptions generate a set of functions of form __cxa_find_matching_catch_1(), __cxa_find_matching_catch_2(), etc.
     // where the number specifies the number of arguments. In Emscripten, route all these to a single function '__cxa_find_matching_catch'
     // that variadically processes all of these functions using JS 'arguments' object.
-    if (item.ident.startsWith('___cxa_find_matching_catch_')) {
+    if (item.identMangled.startsWith('___cxa_find_matching_catch_')) {
       if (DISABLE_EXCEPTION_THROWING) {
         error('DISABLE_EXCEPTION_THROWING was set (likely due to -fno-exceptions), which means no C++ exception throwing support code is linked in, but exception catching code appears. Either do not set DISABLE_EXCEPTION_THROWING (if you do want exception throwing) or compile all source files with -fno-except (so that no exceptions support code is required); also make sure DISABLE_EXCEPTION_CATCHING is set to the right value - if you want exceptions, it should be off, and vice versa.');
         return;
       }
-      var num = +item.ident.split('_').slice(-1)[0];
+      var num = +item.identMangled.split('_').slice(-1)[0];
       addCxaCatch(num);
       // Continue, with the code below emitting the proper JavaScript based on
       // what we just added to the library.
     }
 
-    function addFromLibrary(ident, dependent) {
+    function addFromLibrary(item, dependent) {
+      // dependencies can be JS functions, which we just run
+      if (typeof item == 'function') return item();
+
+      var ident = item.identOrig;
+      var finalName = item.identMangled;
+
       if (ident in addedLibraryItems) return '';
       addedLibraryItems[ident] = true;
 
-      // dependencies can be JS functions, which we just run
-      if (typeof ident == 'function') return ident();
-
       // don't process any special identifiers. These are looked up when processing the base name of the identifier.
       if (isJsLibraryConfigIdentifier(ident)) {
         return '';
       }
 
-      var finalName = mangleCSymbolName(ident);
-
       // if the function was implemented in compiled code, we just need to export it so we can reach it from JS
       if (finalName in IMPLEMENTED_FUNCTIONS) {
         EXPORTED_FUNCTIONS[finalName] = 1;
@@ -170,16 +153,13 @@ function JSify(data, functionsOnly) {
 
       var noExport = false;
 
-      if (allExternPrimitives.indexOf(ident) != -1) {
-        usedExternPrimitives[ident] = 1;
-        return;
-      } else if ((!LibraryManager.library.hasOwnProperty(ident) && !LibraryManager.library.hasOwnProperty(ident + '__inline')) || SIDE_MODULE) {
-        if (!(finalName in IMPLEMENTED_FUNCTIONS) && !(finalName in WEAK_DECLARES) && !LINKABLE) {
+      if (!LibraryManager.library.hasOwnProperty(ident)) {
+        if (!(finalName in IMPLEMENTED_FUNCTIONS) && !LINKABLE) {
           var msg = 'undefined symbol: ' + ident;
           if (dependent) msg += ' (referenced by ' + dependent + ')';
           if (ERROR_ON_UNDEFINED_SYMBOLS) {
             error(msg);
-            if (WASM_BACKEND && !LLD_REPORT_UNDEFINED) {
+            if (!LLD_REPORT_UNDEFINED) {
               warnOnce('Link with `-s LLD_REPORT_UNDEFINED` to get more information on undefined symbols');
             }
             warnOnce('To disable errors for undefined symbols use `-s ERROR_ON_UNDEFINED_SYMBOLS=0`')
@@ -187,6 +167,9 @@ function JSify(data, functionsOnly) {
           } else if (VERBOSE || WARN_ON_UNDEFINED_SYMBOLS) {
             warn(msg);
           }
+          if (ident === 'main' && STANDALONE_WASM) {
+            warn('To build in STANDALONE_WASM mode without a main(), use emcc --no-entry');
+          }
         }
         if (!RELOCATABLE) {
           // emit a stub that will fail at runtime
@@ -196,34 +179,15 @@ function JSify(data, functionsOnly) {
           // (not useful to warn/error multiple times)
           LibraryManager.library[ident + '__docs'] = '/** @type {function(...*):?} */';
         } else {
-          var isGlobalAccessor = ident.startsWith('g$');
-          var realIdent = ident;
-          if (isGlobalAccessor) {
-            realIdent = realIdent.substr(2);
-          }
-
-          var target = (SIDE_MODULE ? 'parent' : '') + "Module['" + mangleCSymbolName(realIdent) + "']";
+          var target = "Module['" + finalName + "']";
           var assertion = '';
           if (ASSERTIONS) {
             var what = 'function';
-            if (isGlobalAccessor) {
-              what = 'global';
-            }
-            assertion += 'if (!' + target + ') abort("external ' + what + ' \'' + realIdent + '\' is missing. perhaps a side module was not linked in? if this function was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment");\n';
+            assertion += 'if (!' + target + ') abort("external symbol \'' + ident + '\' is missing. perhaps a side module was not linked in? if this function was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment");\n';
 
           }
-          var functionBody;
-          if (isGlobalAccessor) {
-            functionBody = assertion + "return " + target + ";"
-          } else {
-            functionBody = assertion + "return " + target + ".apply(null, arguments);";
-          }
+          var functionBody = assertion + "return " + target + ".apply(null, arguments);";
           LibraryManager.library[ident] = new Function(functionBody);
-          if (SIDE_MODULE) {
-            // no dependencies, just emit the thunk
-            Functions.libraryFunctions[finalName] = 1;
-            return processLibraryFunction(LibraryManager.library[ident], ident, finalName);
-          }
           noExport = true;
         }
       }
@@ -236,8 +200,8 @@ function JSify(data, functionsOnly) {
         error('JS library directive ' + ident + '__deps=' + deps.toString() + ' is of type ' + typeof deps + ', but it should be an array!');
         return;
       }
-      deps.forEach(function(dep) {
-        if (typeof snippet === 'string' && !(dep in LibraryManager.library)) warn('missing library dependency ' + dep + ', make sure you are compiling with the right options (see #ifdefs in src/library*.js)');
+      deps.forEach((dep) => {
+        if (typeof snippet === 'string' && !(dep in LibraryManager.library)) warn('missing library dependency ' + dep + ', make sure you are compiling with the right options (see #if in src/library*.js)');
       });
       var isFunction = false;
 
@@ -253,7 +217,7 @@ function JSify(data, functionsOnly) {
           }
           // In asm, we need to know about library functions. If there is a target, though, then no
           // need to consider this a library function - we will call directly to it anyhow
-          if (!redirectedIdent && (typeof target == 'function' || /Math_\w+/.exec(snippet))) {
+          if (!redirectedIdent && (typeof target == 'function')) {
             Functions.libraryFunctions[finalName] = 1;
           }
         }
@@ -284,7 +248,7 @@ function JSify(data, functionsOnly) {
         if (typeof postset === 'function') {
           postset = postset();
         }
-        if (postset && !addedLibraryItems[postsetId] && !SIDE_MODULE) {
+        if (postset && !addedLibraryItems[postsetId]) {
           addedLibraryItems[postsetId] = true;
           itemsDict.GlobalVariablePostSet.push({
             JS: postset + ';'
@@ -298,19 +262,24 @@ function JSify(data, functionsOnly) {
       // In asm, dependencies implemented in C might be needed by JS library functions.
       // We don't know yet if they are implemented in C or not. To be safe, export such
       // special cases.
-      [LIBRARY_DEPS_TO_AUTOEXPORT].forEach(function(special) {
-        deps.forEach(function(dep) {
+      [LIBRARY_DEPS_TO_AUTOEXPORT].forEach((special) => {
+        deps.forEach((dep) => {
           if (dep == special && !EXPORTED_FUNCTIONS[dep]) {
             EXPORTED_FUNCTIONS[dep] = 1;
           }
         });
       });
-      if (VERBOSE) printErr('adding ' + finalName + ' and deps ' + deps + ' : ' + (snippet + '').substr(0, 40));
+      if (VERBOSE) {
+        printErr('adding ' + finalName + ' and deps ' + deps + ' : ' + (snippet + '').substr(0, 40));
+      }
       var identDependents = ident + "__deps: ['" + deps.join("','")+"']";
       function addDependency(dep) {
+        if (typeof dep !== 'function') {
+          dep = {identOrig: dep, identMangled: mangleCSymbolName(dep)};
+        }
         return addFromLibrary(dep, identDependents + ', referenced by ' + dependent);
       }
-      var depsText = (deps ? '\n' + deps.map(addDependency).filter(function(x) { return x != '' }).join('\n') : '');
+      var depsText = (deps ? deps.map(addDependency).filter((x) => x != '').join('\n') + '\n' : '');
       var contentText;
       if (isFunction) {
         // Emit the body of a JS library function.
@@ -321,7 +290,7 @@ function JSify(data, functionsOnly) {
           }
           var sync = proxyingMode === 'sync';
           assert(typeof original === 'function');
-          contentText = modifyFunction(snippet, function(name, args, body) {
+          contentText = modifyFunction(snippet, (name, args, body) => {
             return 'function ' + name + '(' + args + ') {\n' +
                    'if (ENVIRONMENT_IS_PTHREAD) return _emscripten_proxy_to_main_thread_js(' + proxiedFunctionTable.length + ', ' + (+sync) + (args ? ', ' : '') + args + ');\n' + body + '}\n';
           });
@@ -345,28 +314,13 @@ function JSify(data, functionsOnly) {
         contentText = 'var ' + finalName + '=' + snippet + ';';
       }
       var sig = LibraryManager.library[ident + '__sig'];
-      if (isFunction && sig && LibraryManager.library[ident + '__asm']) {
-        // asm library function, add it as generated code alongside the generated code
-        Functions.implementedFunctions[finalName] = sig;
-        asmLibraryFunctions.push(contentText);
-        contentText = ' ';
-        Functions.libraryFunctions[finalName] = 2;
-        noExport = true; // if it needs to be exported, that will happen in emscripten.py
-      }
       // asm module exports are done in emscripten.py, after the asm module is ready. Here
       // we also export library methods as necessary.
       if ((EXPORT_ALL || (finalName in EXPORTED_FUNCTIONS)) && !noExport) {
         contentText += '\nModule["' + finalName + '"] = ' + finalName + ';';
       }
-      if (!LibraryManager.library[ident + '__asm']) {
-        // If we are not an asm library func, and we have a dep that is, then we need to call
-        // into the asm module to reach that dep. so it must be exported from the asm module.
-        // We set EXPORTED_FUNCTIONS here to tell emscripten.py to do that.
-        deps.forEach(function(dep) {
-          if (LibraryManager.library[dep + '__asm']) {
-            EXPORTED_FUNCTIONS[mangleCSymbolName(dep)] = 0;
-          }
-        });
+      if (MAIN_MODULE && sig) {
+        contentText += '\n' + finalName + '.sig = \'' + sig + '\';';
       }
 
       var commentText = '';
@@ -378,24 +332,13 @@ function JSify(data, functionsOnly) {
     }
 
     itemsDict.functionStub.push(item);
-    var shortident = demangleCSymbolName(item.ident);
-    // If this is not linkable, anything not in the library is definitely missing
-    if (item.ident in DEAD_FUNCTIONS) {
-      if (LibraryManager.library[shortident + '__asm']) {
-        warn('cannot kill asm library function ' + item.ident);
-      } else {
-        LibraryManager.library[shortident] = new Function("err('dead function: " + shortident + "'); abort(-1);");
-        delete LibraryManager.library[shortident + '__inline'];
-        delete LibraryManager.library[shortident + '__deps'];
-      }
-    }
-    item.JS = addFromLibrary(shortident, 'top-level compiled C/C++ code');
+    item.JS = addFromLibrary(item, 'top-level compiled C/C++ code');
   }
 
   // Final combiner
 
   function finalCombiner() {
-    var splitPostSets = splitter(itemsDict.GlobalVariablePostSet, function(x) { return x.ident && x.dependencies });
+    var splitPostSets = splitter(itemsDict.GlobalVariablePostSet, (x) => x.ident && x.dependencies);
     itemsDict.GlobalVariablePostSet = splitPostSets.leftIn;
     var orderedPostSets = splitPostSets.splitOut;
 
@@ -419,96 +362,17 @@ function JSify(data, functionsOnly) {
     //
 
     if (!mainPass) {
-      if (!Variables.generatedGlobalBase) {
-        Variables.generatedGlobalBase = true;
-        // Globals are done, here is the rest of static memory
-        if (SIDE_MODULE) {
-          print('gb = alignMemory(getMemory({{{ STATIC_BUMP }}} + ' + MAX_GLOBAL_ALIGN + '), ' + MAX_GLOBAL_ALIGN + ' || 1);\n');
-          // The static area consists of explicitly initialized data, followed by zero-initialized data.
-          // The latter may need zeroing out if the MAIN_MODULE has already used this memory area before
-          // dlopen'ing the SIDE_MODULE.  Since we don't know the size of the explicitly initialized data
-          // here, we just zero the whole thing, which is suboptimal, but should at least resolve bugs
-          // from uninitialized memory.
-          print('for (var i = gb; i < gb + {{{ STATIC_BUMP }}}; ++i) HEAP8[i] = 0;\n');
-        }
-        // emit "metadata" in a comment. FIXME make this nicer
-        print('// STATICTOP = STATIC_BASE + ' + alignMemory(Variables.nextIndexedOffset) + ';\n');
-      }
       var generated = itemsDict.function.concat(itemsDict.type).concat(itemsDict.GlobalVariableStub).concat(itemsDict.GlobalVariable);
-      print(generated.map(function(item) { return item.JS; }).join('\n'));
-
-      if (memoryInitialization.length > 0) {
-        // apply postsets directly into the big memory initialization
-        itemsDict.GlobalVariablePostSet = itemsDict.GlobalVariablePostSet.filter(function(item) {
-          var m;
-          if (m = /^HEAP([\dFU]+)\[([()>\d]+)\] *= *([()|\d{}\w_' ]+);?$/.exec(item.JS)) {
-            var type = getTypeFromHeap(m[1]);
-            var bytes = Runtime.getNativeTypeSize(type);
-            var target = eval(m[2]) << log2(bytes);
-            var value = m[3];
-            try {
-              value = eval(value);
-            } catch(e) {
-              // possibly function table {{{ FT_* }}} etc.
-              if (value.indexOf('{{ ') < 0) return true;
-            }
-            writeInt8s(memoryInitialization, target - Runtime.GLOBAL_BASE, value, type);
-            return false;
-          }
-          return true;
-        });
-        // write out the singleton big memory initialization value
-        if (USE_PTHREADS) {
-          print('if (!ENVIRONMENT_IS_PTHREAD) {') // Pthreads should not initialize memory again, since it's shared with the main thread.
-        }
-        print('/* memory initializer */ ' + makePointer(memoryInitialization, null, 'ALLOC_NONE', 'i8', 'GLOBAL_BASE' + (SIDE_MODULE ? '+H_BASE' : ''), true));
-        if (USE_PTHREADS) {
-          print('}')
-        }
-      } else {
-        print('/* no memory initializer */'); // test purposes
-      }
-
-      if (!SIDE_MODULE && !WASM_BACKEND) {
-        if (USE_PTHREADS) {
-          print('// Pthreads fill their tempDoublePtr memory area into the pthread stack when the thread is run.')
-          // Main thread still statically allocate tempDoublePtr - although it could theorerically also use its stack
-          // (that might allow removing the whole tempDoublePtr variable altogether from the codebase? but would need
-          // more refactoring)
-          print('var tempDoublePtr = ENVIRONMENT_IS_PTHREAD ? 0 : ' + makeStaticAlloc(8) + ';');
-        } else {
-          print('var tempDoublePtr = ' + makeStaticAlloc(8) + ';');
-        }
-        print('\nfunction copyTempFloat(ptr) { // functions, because inlining this code increases code size too much');
-        print('  HEAP8[tempDoublePtr] = HEAP8[ptr];');
-        print('  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];');
-        print('  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];');
-        print('  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];');
-        print('}\n');
-        print('function copyTempDouble(ptr) {');
-        print('  HEAP8[tempDoublePtr] = HEAP8[ptr];');
-        print('  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];');
-        print('  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];');
-        print('  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];');
-        print('  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];');
-        print('  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];');
-        print('  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];');
-        print('  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];');
-        print('}\n');
-      }
-      print('// {{PRE_LIBRARY}}\n'); // safe to put stuff here that statically allocates
-
+      print(generated.map((item) => item.JS).join('\n'));
       return;
     }
 
-    var shellFile = SHELL_FILE ? SHELL_FILE : (SIDE_MODULE ? 'shell_sharedlib.js' : (MINIMAL_RUNTIME ? 'shell_minimal.js' : 'shell.js'));
+    var shellFile = SHELL_FILE ? SHELL_FILE : (MINIMAL_RUNTIME ? 'shell_minimal.js' : 'shell.js');
 
     var shellParts = read(shellFile).split('{{BODY}}');
     print(processMacros(preprocess(shellParts[0], shellFile)));
     var pre;
-    if (SIDE_MODULE) {
-      pre = processMacros(preprocess(read('preamble_sharedlib.js'), 'preamble_sharedlib.js'));
-    } else if (MINIMAL_RUNTIME) {
+    if (MINIMAL_RUNTIME) {
       pre = processMacros(preprocess(read('preamble_minimal.js'), 'preamble_minimal.js'));
     } else {
       pre = processMacros(preprocess(read('support.js'), 'support.js')) +
@@ -525,15 +389,13 @@ function JSify(data, functionsOnly) {
     globalsData = null;
 
     var generated = itemsDict.functionStub.concat(itemsDict.GlobalVariablePostSet);
-    generated.forEach(function(item) { print(indentify(item.JS || '', 2)); });
+    generated.forEach((item) => print(indentify(item.JS || '', 2)));
 
     legalizedI64s = legalizedI64sDefault;
 
-    if (!SIDE_MODULE) {
-      if (USE_PTHREADS) {
-        print('\n // proxiedFunctionTable specifies the list of functions that can be called either synchronously or asynchronously from other threads in postMessage()d or internally queued events. This way a pthread in a Worker can synchronously access e.g. the DOM on the main thread.')
-        print('\nvar proxiedFunctionTable = [' + proxiedFunctionTable.join() + '];\n');
-      }
+    if (USE_PTHREADS) {
+      print('\n // proxiedFunctionTable specifies the list of functions that can be called either synchronously or asynchronously from other threads in postMessage()d or internally queued events. This way a pthread in a Worker can synchronously access e.g. the DOM on the main thread.')
+      print('\nvar proxiedFunctionTable = [' + proxiedFunctionTable.join() + '];\n');
     }
 
     if (!MINIMAL_RUNTIME) {
@@ -546,21 +408,6 @@ function JSify(data, functionsOnly) {
       print(preprocess(read('base64Utils.js')));
     }
 
-    var usedExternPrimitiveNames = Object.keys(usedExternPrimitives);
-    if (usedExternPrimitiveNames.length > 0) {
-      print('// ASM_LIBRARY EXTERN PRIMITIVES: ' + usedExternPrimitiveNames.join(',') + '\n');
-    }
-
-    if (asmLibraryFunctions.length > 0) {
-      print('// ASM_LIBRARY FUNCTIONS');
-      function fix(f) { // fix indenting to not confuse js optimizer
-        f = f.substr(f.indexOf('f')); // remove initial spaces before 'function'
-        f = f.substr(0, f.lastIndexOf('\n')+1); // remove spaces and last }  XXX assumes function has multiple lines
-        return f + '}'; // add unindented } to match function
-      }
-      print(asmLibraryFunctions.map(fix).join('\n'));
-    }
-
     if (abortExecution) throw Error('Aborting compilation due to previous errors');
 
     // This is the main 'post' pass. Print out the generated code that we have here, together with the
@@ -585,21 +432,12 @@ function JSify(data, functionsOnly) {
       print(read('deterministic.js'));
     }
 
-    var postFile = SIDE_MODULE ? 'postamble_sharedlib.js' : (MINIMAL_RUNTIME ? 'postamble_minimal.js' : 'postamble.js');
-    var postParts = processMacros(preprocess(read(postFile), postFile)).split('{{GLOBAL_VARS}}');
-    print(postParts[0]);
-
-    print(postParts[1]);
+    var postFile = MINIMAL_RUNTIME ? 'postamble_minimal.js' : 'postamble.js';
+    var post = processMacros(preprocess(read(postFile), postFile));
+    print(post);
 
     var shellParts = read(shellFile).split('{{BODY}}');
     print(processMacros(preprocess(shellParts[1], shellFile)));
-    // Print out some useful metadata
-    if (RUNNING_JS_OPTS) {
-      var generatedFunctions = JSON.stringify(keys(Functions.implementedFunctions));
-      if (RUNNING_JS_OPTS) {
-        print('// EMSCRIPTEN_GENERATED_FUNCTIONS: ' + generatedFunctions + '\n');
-      }
-    }
 
     PassManager.serialize();
   }
